
/*
 * Copyright (c) Developers of URL Utility
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 *      <date>			        <changed by>		<summary of changes>
 * 13 October 2004  	    Jonee Ryan Ty		First documented history
 *
 */

#include <stdio.h>
#include <ctype.h> // tolower
#include <string.h> // strlen

#define MAX_URL_LEN 1024
	// if MY_VERSION is not zero, it would overwrite an existing file
#define MY_VERSION 0

enum boolean {FALSE, TRUE};

void eatSpaces(FILE *fromFile, int printFirstNonSpace),
    printCurChar(),
    printChar(int c);

int error = 0,  // error flag
    cur,    // currentChar
    hrefPrintOnly,  // TRUE when only one parameter is given
    toAttach,   // TRUE when argc = 4 (ie when a parameter is given to attach to relative urls)
    myArgc;
char *hrefAttach,   // the prefix
    **myArgv;

FILE *output;

int main(int argc, char *argv[]) {
    void read(char *file);
    FILE *tmp;

    myArgc = argc;
    myArgv = argv;

    if (argc == 1) {
        printf("%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
"Program Usage\n",
"-to print this instructions run the program with no additional parameter\n",
"-to print all href and src values in a given html file, run the program with\n"
"    an additional parameter being the file name ie the command:\n",
"       <executable name> index.html\n",
"    prints out all href and src values in index.html\n",
"-to prefix a string to all URL values in a file and the output is in another\n",
"    file do something like this-\n",
"       <executable name> f1.html f2.html http://cookiebreak.tripod.com\n",
"    this would attach http://cookiebreak.tripod.com to all URL values \n",
"    (except those starting with \"www.\", \"http://\", \"ftp://\", \"mailto:\")\n",
"    in f1.html and the output is f2.html\n",
"-to ignore all URLs that starts with certain keywords do something like this\n",
"       <executable name> f1.html f2.html http://cookiebreak.tripod.com k1 k2 kn\n",
"    this would do as the previous command except that it ignores all\n",
"    that starts with k1, k2 and kn\n");

    } if (argc == 2) {    // given just one extra parameter, the program prints out all href and src values
        hrefPrintOnly = TRUE;
        read(argv[1]);
        toAttach = FALSE;

    } else if (argc > 2) {
        if (argc > 3) {
            hrefAttach = argv[3];
            toAttach = TRUE;

        } else if (argc == 3)
            toAttach = FALSE;

        hrefPrintOnly = FALSE;
        tmp = fopen(argv[2], "r");  // to check whether output file exists

        if (tmp != NULL) {
            fclose(tmp);

            if (!MY_VERSION) {
                fprintf(stderr, "file exists: %s\n", argv[2]);
                error = 1;
                return 1;
            }
        }   // no need to close tmp if it is NULL

        output = fopen(argv[2], "w");
        read(argv[1]);
        fclose(output);
    }

    return 0;
} // main

void read(char *file) {

    int handleNextPattern(FILE *fromFile);
    void recordURL(FILE *fromFile, int isQuoted);

    FILE *inputFile;

    inputFile = fopen(file, "r");

    if (inputFile == NULL) {
		fprintf(stderr, "cannot open file: %s\n", file);
		error = 1;
		return;
    }

    while ((cur = getc(inputFile)) != EOF) {
        printCurChar();

        if ((cur == ' ') || (cur == '\n')) {

            if (handleNextPattern(inputFile)) {  // handleNextPattern uses getc

                cur = getc(inputFile);    // this would overwrite the 'f'
                printCurChar();

                eatSpaces(inputFile, TRUE);     // after this, cur is definitely not a white space

                if (cur == '=') {   // the character '=' had already been printed from last function
                    cur = getc(inputFile);

                    eatSpaces(inputFile, FALSE);   // after this, cur holds the first non white space

                    if (cur == '"') {
                        printCurChar();
                        recordURL(inputFile, TRUE);

                    } else if (cur != EOF) //cur currently holds the first letter of the expected URL string
                        recordURL(inputFile, FALSE);
                }
            }
        }
    }

    fclose(inputFile);
} // read

/**
*@param opened file where to check whether cur char is same as second parameter
*/
int handleNextChar(FILE *fromFile, char toCheck, int isCaseSensitive) {
    cur = getc(fromFile);
    printChar(cur);

    if (isCaseSensitive)
        return (cur == toCheck) ? TRUE : FALSE;
    else
        return (tolower(cur) == toCheck) ? TRUE : FALSE;
}

int handleNextPattern(FILE *fromFile) {
    int handleNextChar(FILE *fromFile, char toCheck, int isCaseSensitive),
        a,
        len,
        isPatternMatch = TRUE;
    char *toCheck,
        *pat1 = "href",
        *pat2 = "src";

    eatSpaces(fromFile, TRUE);   // after this, cur holds the first non white space

    if (cur == pat1[0])
        toCheck = pat1;
    else if (cur == pat2[0])
        toCheck = pat2;
    else
        isPatternMatch = FALSE;

    if (isPatternMatch) {
        len = strlen(toCheck);
        for (a = 1; a < len; a++) {
            if (!handleNextChar(fromFile, toCheck[a], FALSE)) {
                a = len; //to terminate loop
                isPatternMatch = FALSE;
            }
        }
    }

    return isPatternMatch;
}

void recordURL(FILE *fromFile, int isQuoted) {
    void handleModURL(char *recURL);

    int digMore = TRUE,
        indx = 0;

    char stop = '"',
        *recURL;

    recURL = (char *)malloc(MAX_URL_LEN * sizeof(char));

    if (!isQuoted) {
        recURL[indx++] = cur;   // cur currently holds the first letter of our expected URL string
        stop = ' ';
    }

    while (digMore) {
        cur = getc(fromFile);
        recURL[indx++] = cur;

        if ((cur == stop) || (cur == EOF)) {
            digMore = FALSE;

            if (cur == ' ')        // allows href=sdfsdfs href=...
                ungetc(cur, fromFile);

        } else if (cur == '\n') {
            if (!isQuoted)
                digMore = FALSE;
        }
    }

    recURL[indx - 1] = '\0';

    handleModURL(recURL);

    free(recURL);
} // recordURL

/**
 * modifies the URL string (ie attach hrefAttach if needed) and do the printing
*/
void handleModURL(char *recURL) {
    int startsWith(char *from, char *pattern, int isCaseSensitive),
        checkForUnchanged(char *from);

    int recIndx = 0,
        attachIndx,
        dotdot, // the number of occurence of "../" in recURL
        recLen,        // recURL
        attachLen,  // hrefAttach
        len,
        i;
    char *newURL;

    if (toAttach) {
        if ((recURL[0] == '#') || (startsWith(recURL, "mailto:", FALSE)) ||
                (startsWith(recURL, "www.", FALSE)) || (startsWith(recURL, "ftp://", FALSE)) ||
                (startsWith(recURL, "http://", FALSE)) || checkForUnchanged(recURL)) {
            printf("%s not changed\n", recURL);
            fprintf(output, "%s%c", recURL, cur);

        } else {
            recLen = strlen(recURL);
            dotdot = 0;
            i = 0;
                /*fix url ie say hrefAttach is http://cookiebreak.tripod.com/styles and recURL is ../index.html*/
            /*count how many ../ recURL starts with*/
            while (i + 2 < recLen) {
                if ((recURL[i] == '.') && (recURL[i + 1] == '.') && (recURL[i + 2] == '/')) {
                    i = i + 3;
                    recIndx = i;
                    dotdot++;
                } else
                    i = recLen; // to terminate loop
            }

                /*fix hrefAttach according to dotdot- find the index of '\' that's desired*/
            attachLen = strlen(hrefAttach);
            attachIndx = attachLen - 1;

            if ((attachLen > 2) && (dotdot > 0)) {
                for (i = attachLen - 2; i > 0; i--) {
                    if (hrefAttach[i] == '/') {
                        dotdot--;
                        if (dotdot == 0) {
                            attachIndx = i;   // variable re-used
                            i = 0;  // to terminate loop
                        }
                    }
                }
            }

            newURL = (char *)malloc((recLen - dotdot * 3 + attachIndx + 2) * sizeof(char));

            for (i = 0; i < attachIndx; i++)
                newURL[i] = hrefAttach[i];
            newURL[attachIndx] = '/';

            len = recLen - dotdot * 3;

            for (i = 0; i < len; i++)
                newURL[attachIndx + 1 + i] = recURL[recIndx + i];
            newURL[attachIndx + len + 1] = '\0';

            printf("%s \t-->>  %s\n", recURL, newURL);
            fprintf(output, "%s%c", newURL, cur);

            free(newURL);
        }

    } else {
        printf("%s\n", recURL);

        if (!hrefPrintOnly)
            fprintf(output, "%s%c", recURL, cur);
    }
} // handleModURL

/**
 * checks whether extra parameter (beyond the fourth including the program name) starts
 * one of the urls
 */
int checkForUnchanged(char *from) {
    int startsWith(char *from, char *pattern, int isCaseSensitive);

    int i, oneMatches;

    if (myArgc < 5)
        oneMatches = FALSE;
    else {
        oneMatches = FALSE;
        for (i = 4; i < myArgc; i++)
            if (startsWith(from, myArgv[i], FALSE)) {
                oneMatches = TRUE;
                i = myArgc;
            }
    }

    return oneMatches;
}

/**
 * checks whether string pattern starts first parameter string
 */
int startsWith(char *from, char *pattern, int isCaseSensitive) {
    int patLen = strlen(pattern),
        doesStart,
        i;

    if (patLen > strlen(from))
        doesStart = FALSE;

    else {
        doesStart = TRUE;

        if (isCaseSensitive) {
            for (i = 0; i < patLen; i++)
                if (from[i] != pattern[i]) {
                    doesStart = FALSE;
                    i = patLen;
                }

        } else {

            for (i = 0; i < patLen; i++)
                if (tolower(from[i]) != tolower(pattern[i])) {
                    doesStart = FALSE;
                    i = patLen;
                }
        }
    }

    return doesStart;
} // startsWith

void eatSpaces(FILE *fromFile, int printFirstNonSpace) {
    if (printFirstNonSpace) { // print until the first non white space
        while (isspace(cur)) {
	        cur = getc(fromFile);
            printCurChar();
        }

    } else {    // don't print the last character
        while (isspace(cur)) {
            printCurChar();
            cur = getc(fromFile);
        }
    }
}

void printChar(int c) {
    if ((c != EOF) && (!hrefPrintOnly))
        fprintf(output, "%c", c);
}

void printCurChar() {
    printChar(cur);
}
